package com.landray.kmss.sys.attachment.integrate.wps.util;

import org.w3c.dom.NodeList;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTText;
import org.apache.xmlbeans.XmlException;
import java.util.Stack;
import java.util.Iterator;
import java.util.List;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.w3c.dom.Node;
import java.util.ArrayList;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTBookmark;

public class BookMark
{
    public static final int INSERT_AFTER = 0;
    public static final int INSERT_BEFORE = 1;
    public static final int REPLACE = 2;
    public static final String RUN_NODE_NAME = "w:r";
    public static final String TEXT_NODE_NAME = "w:t";
    public static final String BOOKMARK_START_TAG = "bookmarkStart";
    public static final String BOOKMARK_END_TAG = "bookmarkEnd";
    public static final String BOOKMARK_ID_ATTR_NAME = "w:id";
    public static final String STYLE_NODE_NAME = "w:rPr";
    public static final String STYLE_NODE_ATTR_NAME = "w:rsidRPr";
    public static final String STYLE_NODE_ATTR_NAME_1 = "w:rsidR";
    private CTBookmark _ctBookmark;
    private XWPFParagraph _para;
    private XWPFTableCell _tableCell;
    private String _bookmarkName;
    private boolean _isCell;
    private String _bookTrackFlag;
    
    public BookMark(final CTBookmark ctBookmark, final XWPFParagraph para, final String trackFlag) {
        this._ctBookmark = null;
        this._para = null;
        this._tableCell = null;
        this._bookmarkName = null;
        this._isCell = false;
        this._bookTrackFlag = "";
        this._ctBookmark = ctBookmark;
        this._para = para;
        this._bookmarkName = ctBookmark.getName();
        this._tableCell = null;
        this._isCell = false;
        this._bookTrackFlag = trackFlag;
    }
    
    public BookMark(final CTBookmark ctBookmark, final XWPFParagraph para, final XWPFTableCell tableCell, final String trackFlag) {
        this(ctBookmark, para, trackFlag);
        this._tableCell = tableCell;
        this._isCell = true;
    }
    
    public boolean isInTable() {
        return this._isCell;
    }
    
    public XWPFTable getContainerTable() {
        return this._tableCell.getTableRow().getTable();
    }
    
    public XWPFTableRow getContainerTableRow() {
        return this._tableCell.getTableRow();
    }
    
    public String getBookmarkName() {
        return this._bookmarkName;
    }
    
    public CTBookmark get_ctBookmark() {
        return this._ctBookmark;
    }
    
    public String get_bookTrackFlag() {
        return this._bookTrackFlag;
    }
    
    public void insertTextAtBookMark(final String bookmarkValue, final int where) {
        if (this._isCell) {
            this.handleBookmarkedCells(bookmarkValue, where);
        }
        else {
            final XWPFRun run = this._para.createRun();
            final String[] multine = bookmarkValue.split("\n");
            if (multine.length <= 1) {
                run.setText(bookmarkValue);
            }
            else {
                run.setText(multine[0]);
            }
            switch (where) {
                case 0: {
                    this.insertAfterBookmark(run);
                    break;
                }
                case 1: {
                    this.insertBeforeBookmark(run);
                    break;
                }
                case 2: {
                    this.replaceBookmark(run);
                    break;
                }
            }
            if (multine.length > 1) {
                Node textStyleNode = null;
                Node paraNode = this._ctBookmark.getDomNode();
                final List<Node> saveNodeList = new ArrayList<Node>();
                boolean saveNode = false;
                while (true) {
                    if (paraNode.getNodeName().contains("bookmarkEnd")) {
                        saveNode = true;
                    }
                    paraNode = paraNode.getNextSibling();
                    if (paraNode == null) {
                        break;
                    }
                    if (saveNode) {
                        saveNodeList.add(paraNode);
                    }
                    else {
                        if (!"w:r".equals(paraNode.getNodeName()) && !"r".equals(paraNode.getNodeName())) {
                            continue;
                        }
                        textStyleNode = this.getStyleNode(paraNode);
                        paraNode = paraNode.getNextSibling();
                    }
                }
                for (int p = 1; p < multine.length; ++p) {
                    final XWPFRun newrun = this._para.createRun();
                    if (textStyleNode != null) {
                        newrun.getCTR().getDomNode().insertBefore(textStyleNode.cloneNode(true), newrun.getCTR().getDomNode().getFirstChild());
                    }
                    newrun.addBreak();
                    newrun.setText(multine[p]);
                }
                final List<Node> appendNodeList = new ArrayList<Node>();
                for (final Node tempNode : saveNodeList) {
                    final Node newNode = tempNode.cloneNode(true);
                    this._para.getCTP().getDomNode().removeChild(tempNode);
                    appendNodeList.add(newNode);
                }
                this.moveBookMarkEnd(appendNodeList);
            }
        }
    }
    
    private void moveBookMarkEnd(final List<Node> appendNodeList) {
        Node nextNode = this._ctBookmark.getDomNode();
        String startBookMarkFlag = "";
        final Node w_id = nextNode.getAttributes().getNamedItem("w:id");
        if (w_id != null) {
            startBookMarkFlag = w_id.getNodeValue();
        }
        nextNode = nextNode.getNextSibling();
        if (nextNode == null) {
            return;
        }
        if (nextNode.getNodeName().contains("bookmarkEnd")) {
            final Node end_w_id = nextNode.getAttributes().getNamedItem("w:id");
            if (end_w_id != null) {
                final String endBookMarkFlag = end_w_id.getNodeValue();
                if (endBookMarkFlag.equals(startBookMarkFlag)) {
                    return;
                }
            }
        }
        Node endBookMark = null;
        boolean hasNextRun = false;
        while (true) {
            nextNode = nextNode.getNextSibling();
            if (nextNode == null) {
                break;
            }
            if (nextNode.getNodeName().contains("bookmarkEnd")) {
                final Node end_w_id2 = nextNode.getAttributes().getNamedItem("w:id");
                if (end_w_id2 != null) {
                    final String endBookMarkFlag2 = end_w_id2.getNodeValue();
                    if (endBookMarkFlag2.equals(startBookMarkFlag)) {
                        endBookMark = nextNode;
                        continue;
                    }
                }
            }
            if (endBookMark != null) {
                hasNextRun = true;
            }
            if (!nextNode.getNodeName().contains("w:r")) {
                break;
            }
        }
        if (hasNextRun && endBookMark != null) {
            final Node newEndNode = endBookMark.cloneNode(true);
            this._para.getCTP().getDomNode().removeChild(endBookMark);
            this._para.getCTP().getDomNode().appendChild(newEndNode);
        }
        for (final Node addNode : appendNodeList) {
            this._para.getCTP().getDomNode().appendChild(addNode);
        }
    }
    
    private void insertAfterBookmark(final XWPFRun run) {
        Node nextNode = null;
        Node insertBeforeNode = null;
        Node styleNode = null;
        int bookmarkStartID = 0;
        int bookmarkEndID = -1;
        bookmarkStartID = this._ctBookmark.getId().intValue();
        nextNode = this._ctBookmark.getDomNode();
        while (bookmarkStartID != bookmarkEndID) {
            nextNode = nextNode.getNextSibling();
            if (nextNode.getNodeName().contains("bookmarkEnd")) {
                try {
                    bookmarkEndID = Integer.parseInt(nextNode.getAttributes().getNamedItem("w:id").getNodeValue());
                }
                catch (final NumberFormatException ex) {
                    bookmarkEndID = bookmarkStartID;
                }
            }
            else {
                if (!nextNode.getNodeName().equals("w:r")) {
                    continue;
                }
                styleNode = this.getStyleNode(nextNode);
            }
        }
        insertBeforeNode = nextNode.getNextSibling();
        if (styleNode != null) {
            run.getCTR().getDomNode().insertBefore(styleNode.cloneNode(true), run.getCTR().getDomNode().getFirstChild());
        }
        if (insertBeforeNode != null) {
            this._para.getCTP().getDomNode().insertBefore(run.getCTR().getDomNode(), insertBeforeNode);
        }
    }
    
    private void insertBeforeBookmark(final XWPFRun run) {
        Node insertBeforeNode = null;
        Node childNode = null;
        Node styleNode = null;
        insertBeforeNode = this._ctBookmark.getDomNode();
        childNode = insertBeforeNode.getPreviousSibling();
        if (childNode != null) {
            styleNode = this.getStyleNode(childNode);
            if (styleNode != null) {
                run.getCTR().getDomNode().insertBefore(styleNode.cloneNode(true), run.getCTR().getDomNode().getFirstChild());
            }
        }
        this._para.getCTP().getDomNode().insertBefore(run.getCTR().getDomNode(), insertBeforeNode);
    }
    
    private void replaceBookmark(final XWPFRun run) {
        Node nextNode = null;
        Node styleNode = null;
        Node lastRunNode = null;
        Stack<Node> nodeStack = null;
        int bookmarkStartID = 0;
        int bookmarkEndID = -1;
        nodeStack = new Stack<Node>();
        bookmarkStartID = this._ctBookmark.getId().intValue();
        nextNode = this._ctBookmark.getDomNode();
        String startBookMarkFlag = "";
        final Node w_id = nextNode.getAttributes().getNamedItem("w:id");
        if (w_id != null) {
            startBookMarkFlag = w_id.getNodeValue();
        }
        nodeStack.push(nextNode);
        while (bookmarkStartID != bookmarkEndID) {
            nextNode = nextNode.getNextSibling();
            if (nextNode == null) {
                bookmarkEndID = bookmarkStartID;
                break;
            }
            nodeStack.push(nextNode);
            if (!nextNode.getNodeName().contains("bookmarkEnd")) {
                continue;
            }
            final Node end_w_id = nextNode.getAttributes().getNamedItem("w:id");
            if (end_w_id == null) {
                continue;
            }
            final String endBookMarkFlag = end_w_id.getNodeValue();
            if (!endBookMarkFlag.equals(startBookMarkFlag)) {
                continue;
            }
            try {
                bookmarkEndID = Integer.parseInt(nextNode.getAttributes().getNamedItem("w:id").getNodeValue());
            }
            catch (final NumberFormatException ex) {
                bookmarkEndID = bookmarkStartID;
            }
        }
        for (int i = 0; i < nodeStack.size(); ++i) {
            final Node curNode = nodeStack.elementAt(i);
            if ("w:ins".equals(curNode.getNodeName())) {
                final Node childNode = curNode.getFirstChild();
                if (childNode != null && "w:r".equals(childNode.getNodeName())) {
                    styleNode = this.getStyleNode(childNode);
                    if (styleNode != null) {
                        run.getCTR().getDomNode().insertBefore(styleNode.cloneNode(true), run.getCTR().getDomNode().getFirstChild());
                        break;
                    }
                }
            }
            else if ("w:r".equals(curNode.getNodeName())) {
                styleNode = this.getStyleNode(curNode);
                if (styleNode != null) {
                    run.getCTR().getDomNode().insertBefore(styleNode.cloneNode(true), run.getCTR().getDomNode().getFirstChild());
                    break;
                }
            }
        }
        if (!nodeStack.isEmpty()) {
            lastRunNode = nodeStack.peek();
            if (lastRunNode.getNodeName().equals("w:r")) {
                styleNode = this.getStyleNode(lastRunNode);
                if (styleNode != null) {
                    run.getCTR().getDomNode().insertBefore(styleNode.cloneNode(true), run.getCTR().getDomNode().getFirstChild());
                }
            }
            this.deleteChildNodes(nodeStack);
        }
        this._para.getCTP().getDomNode().insertBefore(run.getCTR().getDomNode(), nextNode);
    }
    
    private void deleteChildNodes(final Stack<Node> nodeStack) {
        Node toDelete = null;
        int bookmarkStartID = 0;
        int bookmarkEndID = 0;
        boolean inNestedBookmark = false;
        for (int i = 1; i < nodeStack.size(); ++i) {
            toDelete = nodeStack.elementAt(i);
            if (toDelete.getNodeName().contains("bookmarkStart")) {
                bookmarkStartID = Integer.parseInt(toDelete.getAttributes().getNamedItem("w:id").getNodeValue());
                inNestedBookmark = true;
            }
            else if (toDelete.getNodeName().contains("bookmarkEnd")) {
                bookmarkEndID = Integer.parseInt(toDelete.getAttributes().getNamedItem("w:id").getNodeValue());
                if (bookmarkEndID == bookmarkStartID) {
                    inNestedBookmark = false;
                }
            }
            else if (!inNestedBookmark) {
                this._para.getCTP().getDomNode().removeChild(toDelete);
            }
        }
    }
    
    private Node getStyleNode(final Node parentNode) {
        Node childNode = null;
        Node styleNode = null;
        if (parentNode != null && (parentNode.getNodeName().equalsIgnoreCase("w:r") || "r".equals(parentNode.getNodeName())) && parentNode.hasChildNodes()) {
            childNode = parentNode.getFirstChild();
            if ("w:rPr".equals(childNode.getNodeName())) {
                styleNode = childNode;
            }
            else {
                while ((childNode = childNode.getNextSibling()) != null) {
                    if (childNode.getNodeName().equals("w:rPr")) {
                        styleNode = childNode;
                        childNode = null;
                    }
                }
            }
        }
        return styleNode;
    }
    
    public String getBookmarkText() throws XmlException {
        StringBuilder builder = null;
        if (this._tableCell != null) {
            builder = new StringBuilder(this._tableCell.getText());
        }
        else {
            builder = this.getTextFromBookmark();
        }
        return (builder == null) ? null : builder.toString();
    }
    
    private StringBuilder getTextFromBookmark() throws XmlException {
        int startBookmarkID = 0;
        int endBookmarkID = -1;
        Node nextNode = null;
        StringBuilder builder = null;
        startBookmarkID = this._ctBookmark.getId().intValue();
        nextNode = this._ctBookmark.getDomNode();
        builder = new StringBuilder();
        while (startBookmarkID != endBookmarkID) {
            nextNode = nextNode.getNextSibling();
            if (nextNode == null) {
                endBookmarkID = startBookmarkID;
                break;
            }
            if (nextNode.getNodeName().contains("bookmarkEnd")) {
                try {
                    endBookmarkID = Integer.parseInt(nextNode.getAttributes().getNamedItem("w:id").getNodeValue());
                }
                catch (final NumberFormatException ex) {
                    endBookmarkID = startBookmarkID;
                }
            }
            else {
                if (!nextNode.getNodeName().equals("w:r") || !nextNode.hasChildNodes()) {
                    continue;
                }
                builder.append(this.getTextFromChildNodes(nextNode));
            }
        }
        return builder;
    }
    
    private String getTextFromChildNodes(final Node node) throws XmlException {
        NodeList childNodes = null;
        Node childNode = null;
        CTText text = null;
        final StringBuilder builder = new StringBuilder();
        int numChildNodes = 0;
        childNodes = node.getChildNodes();
        numChildNodes = childNodes.getLength();
        for (int i = 0; i < numChildNodes; ++i) {
            childNode = childNodes.item(i);
            if (childNode.getNodeName().equals("w:t")) {
                if (childNode.getNodeType() == 3) {
                    builder.append(childNode.getNodeValue());
                }
                else {
                    text = (CTText)CTText.Factory.parse(childNode);
                    builder.append(text.getStringValue());
                }
            }
        }
        return builder.toString();
    }
    
    private void handleBookmarkedCells(final String bookmarkValue, final int where) {
        final XWPFRun run = this._para.createRun();
        run.setText(bookmarkValue);
        switch (where) {
            case 0: {
                this.insertAfterBookmark(run);
                break;
            }
            case 1: {
                this.insertBeforeBookmark(run);
                break;
            }
            case 2: {
                this.replaceBookmark(run);
                break;
            }
        }
    }
    
    public XWPFTableCell getContainerTableCell() {
        return this._tableCell;
    }
}
